package corbaTrace.log2xmi.messages;

import java.util.*;
import corbaTrace.utils.IndentString;

/**
 *
 * @author Nicolas Lemoullec
 * @author Antoine Parra del Pozo
 */
public class Message {
    protected String messageId;
    protected short messageType;    // even numbers are receptions, odd numbers are emissions.
    protected String sourceObjectId;
    protected String destinationObjectId;
    protected String sendingMessageDate;
    protected String receivingMessageDate;
    protected Vector messageOptionList;

    public static final short RECEIVE_REQUEST = 0;
    public static final short SEND_REQUEST = 1;
    public static final short RECEIVE_REPLY = 2;
    public static final short SEND_REPLY = 3;
    public static final short SEND_EXCEPTION = 5;
    
    public Message() {
	messageId = null;
	messageType = -1;
	sourceObjectId = null;
	destinationObjectId = null;
	sendingMessageDate = null;
	receivingMessageDate = null;
	messageOptionList = null;
    }

    
    //===========================================================
    // Access Methods
    //===========================================================

    public String getMessageId() {
	return messageId;
    }
    
    /** @return -1 if not set correctly */
    public short getMessageType() {
	return messageType;
    }
    
    public String getDestinationObjectId() {
	return destinationObjectId;
    }
    
    public String getSourceObjectId() {
	return sourceObjectId;
    }
    
    public String getSendingMessageDate() {
	return sendingMessageDate;
    }
    
    public String getReceivingMessageDate() {
	return receivingMessageDate;
    }
    
    public Vector getMessageOptionList() {
	return messageOptionList;
    }
    
    public MessageOption getMessageOption(String name) {
	for (int i=0; i<this.messageOptionList.size(); ++i) {
	    MessageOption opt = ((MessageOption)(this.messageOptionList.elementAt(i)));
	    if (name.equalsIgnoreCase(opt.getName())) {
		return opt;
	    }
	}
	return ((MessageOption)null);
    }

    /** 
     * @return the name of the operation for this message
     */
    public String getOperationName() {
	// locate the right Option.
	MessageOption operation = getOption("operation");
	if (operation != null) {
	    // gets its attribute named "name" (the name of this operation)
	    return((String)operation.getPropertyValue("name"));  // null if not defined
	} else {
	    return null; // operation is not defined
	}
    }

    
    /** 
     * @param name is the name of the option we're looking for.
     * @return true if this option is defined.
     */
    public boolean optionIsDefined(String name) {
        int cpt = messageOptionList.size() - 1;
	boolean exit = false;
        while (!exit && (cpt >= 0)) {
	    exit = (name.equals(((MessageOption)(messageOptionList.elementAt(cpt))).getName()));
	    cpt--;
        }
        return (exit);
    }

    /** 
     * @param name is the name of the option we're looking for.
     * @return the MessageOption;  null if this option is not found.
     */
    public MessageOption getOption(String name) {
        int cpt = this.messageOptionList.size() - 1;
	boolean exit = false;
        while (!exit && (cpt >= 0)) {
	    exit = (name.equals(((MessageOption)(messageOptionList.elementAt(cpt))).getName()));
	    cpt--;
        }
	if (exit) // option found
	    return (MessageOption)messageOptionList.elementAt(++cpt);
	else
	    return null;
    }



    //===========================================================
    // Modification Methods
    //===========================================================

    public void setMessageId(String messageId) {
	this.messageId = messageId;
    }
    
    public void setMessageType(short val) {
	if ((val != RECEIVE_REQUEST) && (val != SEND_REQUEST)
	    && (val != RECEIVE_REPLY) && (val != SEND_REPLY) 
	    && (val != SEND_EXCEPTION))
	    messageType = -1;
	else
	    messageType = val;
    }
    
    public void setMessageType(String type) {
	if (type.equalsIgnoreCase("RECEIVE_REQUEST")) {
	    messageType = RECEIVE_REQUEST;
	} else if (type.equalsIgnoreCase("RECEIVE_REPLY")) {
	    messageType = RECEIVE_REPLY;
	} else if (type.equalsIgnoreCase("SEND_REQUEST")) {
	    messageType = SEND_REQUEST;
	} else if (type.equalsIgnoreCase("SEND_REPLY")) {
	    messageType = SEND_REPLY;
        } else {
	    messageType = SEND_EXCEPTION;
	}
    }

    public void setDestinationObjectId(String id) {
	destinationObjectId = id;
    }
    
    public void setSourceObjectId(String id) {
	sourceObjectId = id;
    }
    
    public void setSendingMessageDate(String date) {
	sendingMessageDate = date;
    }
    
    public void setReceivingMessageDate(String date) {
	receivingMessageDate = date;
    }
    
    public void addMessageOption(MessageOption m) {
	if (messageOptionList == null)
	    messageOptionList = new Vector();
	messageOptionList.add(m);
    }


    //===========================================================
    // Utility Methods
    //===========================================================

    public String messageTypeToString(short type) {
	switch (type) {
	case RECEIVE_REQUEST : return ("receive_request");
	case RECEIVE_REPLY : return ("receive_reply");
	case SEND_REQUEST : return ("send_request");
	case SEND_REPLY : return ("send_reply");
	case SEND_EXCEPTION : return ("send_exception");
	default : return("");
	}
    }

    public String toString() {
	return toString(new IndentString());
    }

    public String toString(IndentString s) {
	s.append("\n"); s.indent(); s.append("<message ");
        s.append("IDmessage=\""+messageId+"\" ");
        s.append("type=\""+messageTypeToString(messageType)+"\">");
	s.inc();
	s.append("\n"); s.indent(); s.append("<local_object IDobject=\""+sourceObjectId+"\" date=\""+sendingMessageDate+"\" />");
	s.append("\n"); s.indent(); s.append("<distant_object IDobject=\"");
	if (destinationObjectId!=null)
	    s.append(destinationObjectId);
	s.append("\" date=\"");
	if (receivingMessageDate!=null)
	    s.append(receivingMessageDate);
	s.append("\" />");

	// print its options
	if (messageOptionList != null) {
	    s.append("\n"); s.indent(); s.append("<options>");
	    s.inc();
	    for (int i=0; i<messageOptionList.size(); ++i) {
		((MessageOption)messageOptionList.elementAt(i)).toString(s);
	    }	
	    s.dec();
	    s.append("\n"); s.indent(); s.append("</options>");
	}
	s.dec();
	s.append("\n"); s.indent(); s.append("</message>\n");
	return s.toString();
    }
    
}
