/**
 * This file is a part of the project : CorbaTrace
 *
 * It's under LGPL licence.
 */

package corbaTrace.interceptorCore;

import java.io.*;
import java.util.*;
import java.lang.*;
import java.text.*;
import org.omg.CORBA.*;
import org.omg.Dynamic.*;
import org.omg.DynamicAny.*;
import org.omg.DynamicAny.DynAnyPackage.*;
import org.omg.DynamicAny.DynAnyFactoryPackage.*;
import org.omg.PortableInterceptor.*;
import org.omg.PortableInterceptor.ORBInitInfoPackage.*;
import org.omg.IOP.Codec;

import corbaTrace.utils.*;

/** abstract class for interceptor logging facilities 
 * @author Etienne Juliot
 * @version 0.1
 */
public abstract class RequestInterceptorLogger
    extends LocalObject {


     /** ==================================================================<br/>
     *<br/>
     *  Attributes & Constructors<br/>
     *<br/>
     *<br/>
     * note : we shoudn't define attributes & constructors here<br/>
     *        because it's an abstract class.<br/>
     *        But, as our Server & Client Interceptor Loggers<br/>
     *        are based on the same constructeur model, and as<br/>
     *        Java allows to define attributes/constructors even in<br/>
     *        abstract classes, we do it this way to make the code more clear.<br/>
     * ==================================================================
     */

    //
    // Attributes
    //

    //
    // Dfinition d'un tableau de flux de sortie
    //
  

    // Used to encode/decode the call stack information so that
    // it can be used as the service context
      protected Codec cdrCodec_;

    // slot id
      protected int mySlotId_;
    public static int slotId;

    // for identifying client or server.
    protected DynAnyFactory dynAnyFactoryS_;

    // for converting any in a specific type
    protected AnyConverter anyConverter;

    
    // ------------------------------------------------------------------------------


    //
    // Constants
    //

    // Service context identifiers
      protected final int REQUEST_CONTEXT_ID = 100;
      protected final int REPLY_CONTEXT_ID = 101;

    // Date format FULL US
    protected final DateFormat df = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZZ");
    //DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.LONG);

    // ID unique
    protected int id_unique = 0;

    // ------------------------------------------------------------------------------


    /**
     * constructor
     */

    public RequestInterceptorLogger(ORBInitInfo info, int slotId) {
	mySlotId_ = slotId;
	this.slotId = slotId;

	//
	// Get the codec factory
	//
	org.omg.IOP.CodecFactory factory = info.codec_factory();
	if(factory == null)
	    throw new RuntimeException();

	//
	// Create codec
	//
	org.omg.IOP.Encoding how = new org.omg.IOP.Encoding();
	how.major_version = 1;
	how.minor_version = 0;
	how.format = org.omg.IOP.ENCODING_CDR_ENCAPS.value;

	try
	{
	    cdrCodec_ = factory.create_codec(how);
	}
	catch(org.omg.IOP.CodecFactoryPackage.UnknownEncoding ex)
	{
	    throw new RuntimeException();
	}
	if(cdrCodec_ == null)
	    throw new RuntimeException();

	//
	// Get the dynamic any factory
	//
	DynAnyFactory dynAnyFactory = null;
	try
	{
	    org.omg.CORBA.Object obj = 
		info.resolve_initial_references("DynAnyFactory");
	    dynAnyFactory = DynAnyFactoryHelper.narrow(obj);
	}
	catch(InvalidName ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    throw new RuntimeException();
	}

	dynAnyFactoryS_ = dynAnyFactory;
	anyConverter = new AnyConverter(dynAnyFactoryS_);
    } // end constructor



    // ==================================================================
    // ==================================================================



    /** 
     *    RequestInterceptorLogger offers various methods for generating a log file.
     */

    private void insertHeader(IndentString out, 
			      RequestInfo info, 
			      String type) {
	// get current date and converts it as String using a defined date format (df).
	Date dt = new Date();
	String laDate = df.format(dt);
	
	// insert the message type, the date and time.
	out.insert("<message mesg_id=\"" + nextId() + "\" " +  
		   "request_id=\"" + info.request_id() + "\" " + 
		   "type=\"" + type + "\">");

	// add an indent level.
	out.inc();

	//out.insert("<message_id 

	out.insert("<local_object id=\"" + getLocalObjectId(info) + 
		   "\" date=\"" + laDate + "\"/>");

	insertDistantObject(out, info);
	out.newLine();
    }

    // ------------------------------------------------------------------------------

    private void insertArguments(IndentString out, 
				 RequestInfo info) {
	//
	// Display arguments
	//
	try
	{
	    Parameter[] args = info.arguments();

	    //out.insert("<arguments>");
	    out.inc();    // increments indent level for next inserts.

	    if (args.length != 0){
                // for each argument
		for (int i = 0 ; i < args.length ; i++){
		    out.indent();

		    // 1) add argument's I/O status : in, out, inout
                    out.append("<argument inout=\"");
		    if(args[i].mode == org.omg.CORBA.ParameterMode.PARAM_IN){
			out.append("in");
		    } else if(args[i].mode == org.omg.CORBA.ParameterMode.PARAM_OUT){
			out.append("out");
		    } else {  // case PARAM_INOUT
			out.append("inout");
		    }

		    // 2) add argument's value
		    out.append("\" value=\"");
		    Any clientName = args[i].argument;

		    out.append(anyConverter.getValue(clientName));
		    out.append("\"");

		    // 3) add argument's type
		    out.append(" type=\"");
		    out.append(anyConverter.getValue(clientName.type()));
		    out.append("\"/>");
		    out.newLine();
		}
	    }
	    
	    out.dec();  // decrements indent level.
	    //out.insert("</arguments>");
	}
	catch(BAD_INV_ORDER ex)
	{
            System.err.println("\nOperation not supported in this context : " + ex + "\n");
	}
	catch(NO_RESOURCES ex)
	{
            System.err.println("\nOperation not supported in this environnement : " + ex + "\n");
	}

    }


    // ------------------------------------------------------------------------------

    protected void insertSendingException(IndentString out, 
					  RequestInfo info) {
	//
	// Display sending exception
	//  => must be redefined by the server logging class.
    }


        
    void insertException(IndentString out, 
			 RequestInfo info) {

	//
	// Display exceptions
	//
	try
	{
	    org.omg.CORBA.TypeCode[] exceptions = info.exceptions();

	    if (exceptions.length == 0)
	    {
		out.insert("<exceptions/>");
	    }
	    else
	    {
		out.insert("<exceptions>");
		out.inc();
		
		for (int i = 0 ; i < exceptions.length ; i++) {
		    out.insert("<exception>");
		    out.insert(anyConverter.getValue(exceptions[i]));
		    out.insert("</exception>");
		}
		out.dec();
		out.insert("</exceptions>");
	    }
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	catch(org.omg.CORBA.NO_RESOURCES ex)
	{
	    // Operation not supported in this environment
	}

    }

    void insertResult(IndentString out, 
			 RequestInfo info) {
	
	//
	// Display result
	//
	try
	{
	    org.omg.CORBA.Any result = info.result();

	    out.indent();
	    out.append("<result value=\"");
	    out.append(anyConverter.getValue(result));
	    out.append("\"/>");
	    out.newLine();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	catch(org.omg.CORBA.NO_RESOURCES ex)
	{
	    // Operation not supported in this environment
	}

    }

    void insertResponseExpected(IndentString out, 
				RequestInfo info) {
	

	//
	// Display response expected
	//
	boolean response_expected = info.response_expected();

	out.insert("<responseExpected value=\"" + 
		   String.valueOf(response_expected) + "\"/>");
	
    }


    void insertReplyStatus(IndentString out, 
			    RequestInfo info) {
	

	//
	// Display reply status
	//
	try
	{
	    short status = info.reply_status();

	    out.indent();
	    out.append("<reply status=\"");
	    if (status == org.omg.PortableInterceptor.SUCCESSFUL.value)
		out.append("successful");
	    else if (status == org.omg.PortableInterceptor.
		     SYSTEM_EXCEPTION.value)
		out.append("system_exception");
	    else if (status == org.omg.PortableInterceptor.
		     USER_EXCEPTION.value)
		out.append("user_exception");
	    else if (status == org.omg.PortableInterceptor.
		     LOCATION_FORWARD.value)
		out.append("location_forward");
	    else if (status == org.omg.PortableInterceptor.
		     LOCATION_FORWARD_PERMANENT.value)
		out.append("location_forward_permanent");
	    else if (status == org.omg.PortableInterceptor.
		     TRANSPORT_RETRY.value)
		out.append("transport_retry");
	    else
		out.append("unknown_reply_status");
	    out.append("\"/>");
	    out.newLine();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}


    }



    // ------------------------------------------------------------------------------

    private void insertFoot(IndentString out) {
	// just close the message tag, and jump a line.
	out.dec();   // decrements indent level.
	out.insert("</message>");

	out.newLine();
	out.newLine();
    }


    // ------------------------------------------------------------------------------

    private void insertOptions(IndentString out, 
			       RequestInfo info,
			       short logLevel) {
	out.newLine();
	out.insert("<options>");
	out.inc();
	insertException(out, info);

	insertResponseExpected(out, info);

	insertReplyStatus(out, info);

	
        // inserts sending informations (for Servers only)
	insertSendingException(out, info);

        if (logLevel > 2)
	    // inserts object ID, Display Adapter Id, etc.
	    insertLogDetail(out, info);

	out.dec();
	out.insert("</options>");
    }



    // ------------------------------------------------------------------------------

    //
    // Display PortableInterceptor::RequestInfo
    // This method gets and write intercepted messages in a log file.
    //
    protected void displayRequestInfo(RequestInfo info, 
				      String type) {

	if(isLogActivate(info)) {
	    short logLevel = getLogLevel(info);
	    
	    // creates a new IndentString for this RequestInfo
	    IndentString out = new IndentString();
	    
	    // inserts Header with date, time, and IDs informations.
	    insertHeader(out, info, type);
	    
	    // inserts current operation and its arguments.
	    out.insert("<operation name=\"" + info.operation() + "\">");
	    insertArguments(out, info);
	    out.insert("</operation>");
	    
	    insertResult(out, info);
	    
	    insertOptions(out, info, logLevel);
	    
	    
	    // inserts Foot
	    insertFoot(out);
	    
	    writeInFile(getNameLogFile(info), out.toString());
	}
	    
    } // End DisplayRequestInfo()


    // -----------------------------------------------------------------



    protected void writeInFile(String nameFile, String text) {
	try {
	    FileWriter fileOut = null;
	    File out = new File(nameFile + "trace.xml");
	    if (out.createNewFile()) {
		System.out.println("Log file " + nameFile + " created");
		/*text = "<?xml version=\"1.0\" standalone=\"no\"?>\n" +
		    "<!DOCTYPE log SYSTEM \"etc/log.dtd\">\n" + 
		    "<log>\n\n" + text;*/
	    }
	    synchronized (out){
		fileOut = new FileWriter(out.toString(), true);
	    	fileOut.write(text);
		fileOut.flush();
	    } // fin de "synchronized(out)"
	} catch(IOException e) {
	    System.err.println("Writing error on file " + nameFile);
	}
    }

    protected abstract void insertDistantObject(IndentString out, 
						RequestInfo info);
	
    protected abstract String getNameLogFile(RequestInfo info);

    protected abstract String getServerId(RequestInfo info);

    protected abstract String getLocalObjectId(RequestInfo info);
    
    protected abstract String getDistantObjectId(RequestInfo info);
    
    protected abstract short getLogLevel(RequestInfo info);

    protected boolean isLogActivate(RequestInfo info) {
	short l = getLogLevel(info);
	if(l > 0)
	    return true;
	else
	    return false;
    }

    protected String getClientId(RequestInfo info) {
	String id;
	try {
	    id = getDataflow(info).client_id;
	}catch(org.omg.CORBA.BAD_OPERATION e) {
	    id = "Unknown";
	}
	return(id);// return "truc";
    }
//protected abstract String getClientId(RequestInfo info);

    protected abstract void insertLogDetail(IndentString out, RequestInfo info);

    /*private String getMessageId(RequestInfo info) {
	return String.valueOf(info.request_id() + " " + nextId() + " " + getDataflow(info).request_id);
    }*/

    protected Dataflow getDataflow(RequestInfo info) 
	throws org.omg.CORBA.BAD_OPERATION {

	try {
	    Any any = info.get_slot(mySlotId_);
	    if( any == null )
		throw new org.omg.CORBA.BAD_OPERATION();
	    Dataflow flow = DataflowHelper.extract(any);
	    return flow;
	} catch(org.omg.PortableInterceptor.InvalidSlot e) {
	    throw new org.omg.CORBA.BAD_OPERATION(e.getMessage());
	}
    }

    protected String getDistantDate(RequestInfo info) {
        try {
	  long date = getDataflow(info).date_send;
	  return df.format(new Date(date));
	} catch(org.omg.CORBA.BAD_OPERATION e) {
	  return null;
	}
    }

    protected Any createAny() 
	throws org.omg.DynamicAny.DynAnyFactoryPackage.InconsistentTypeCode{
	org.omg.DynamicAny.DynAny dynAny = dynAnyFactoryS_.create_dyn_any_from_type_code(DataflowHelper.type());
	return dynAny.to_any();
		    
    }

    protected int nextId() {
	return id_unique++;
    }

}
